use std::collections::HashSet;
use std::fs;

pub fn solve() {
    let path = "res/06/input.txt";

    let contents = fs::read_to_string(path).expect("I/O error, wrong path?");

    // Parse chars, filter out newlines
    let chars: Vec<char> = contents.chars().filter(|c| *c != '\n').collect();

    // Task 1
    let res = find_marker(4, &chars);
    println!("Result 1: {}", res);

    // Task 2
    let res = find_marker(14, &chars);
    println!("Result 2: {}", res);
}

// Finds 1st occurence of l unique characters in chars and returns
// the index of the last of them or -1
fn find_marker(l: usize, chars: &Vec<char>) -> i32 {
    for i in l - 1..chars.len() {
        let slice = &chars[i - (l - 1)..=i];
        let mut set: HashSet<char> = HashSet::new();
        slice.iter().for_each(|c| {
            set.insert(*c);
        });
        if set.len() == l {
            return (i + 1) as i32;
        }
    }

    -1
}

fn _find_marker_v2(l: usize, chars: &Vec<char>) -> usize {
    chars
        .iter()
        .filter(|c| **c != '\n')
        .collect::<Vec<&char>>()
        .windows(l)
        .position(|window| window.iter().collect::<HashSet<_>>().len() == l)
        .map(|pos| pos + l)
        .unwrap()
}
